// Problème des N-reines
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

bool** board_alloc(int n) {
   bool** tab = malloc(n*sizeof(bool*));
   for (int i=0;i<n;i++) {
      tab[i] = malloc(n*sizeof(bool));
      for (int j=0;j<n;j++) {
         tab[i][j] = true;
      }
   }
   return tab;
}

void board_free(int n,bool** tab) {
   for (int i=0;i<n;i++) {
      free(tab[i]);
   }
   free(tab);
}

bool** clone(int n,bool** board) {
   bool** tab = board_alloc(n);
   for (int i=0;i<n;i++) {
      for (int j=0;j<n;j++) {
         tab[i][j] = board[i][j];
      }
   }
   return tab;
}

// Retourne une copie du tableau <board> complété avec les positions 
// prises sur la droite droite par une reine placée en <board(li,co)>
bool** prises_devant(int n,bool** board,int li,int co) {
   bool** cases_prises = clone(n,board);
   cases_prises[li][co] = false; // position de la reine
   for (int j=1;j<n-co;j++) {
      // horizontale et diagonales à droite de la reine
      if (j <= li) {
         cases_prises[li-j][co+j] = false;
      }
      cases_prises[li][co+j] = false;
      if (li+j < n) {
         cases_prises[li+j][co+j] = false;
      }
   }
   return cases_prises;
}

// Calcule le nombre de solutions au problème des <N> reines
void nb_sol(int n,bool** board,int co, int* ptr_cpt) {
   for (int li=0;li<n;li++) {
      if (board[li][co]) {
         if (co < n-1) {
            bool** tab = prises_devant(n,board,li,co);
            nb_sol(n,tab,co+1,ptr_cpt);
            board_free(n,tab);
         } else {
            *ptr_cpt = (*ptr_cpt)+1;
         }
      }
   }
} 

void main() {
   int n = 8;
   // échiquier où placer les reines
   bool** board = board_alloc(n);
   // compteur du nombre de solutions au problème des <N> reines
   int cpt = 0; 
   nb_sol(n,board,0,&cpt);
   printf("Nombre de solutions: %d\n",cpt);
}
